local SM = AceLibrary("SharedMedia-1.0")
local Table={}
Relog.Table=Table

local Columns={}

function Table:SetupDivider(Divider)
	Divider:SetMovable(true)
	Divider:EnableMouse(true)
	
	Divider:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
		
	Divider:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
							this:StopMovingOrSizing();
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+1,0)							
							this.isMoving = false;
						end
						end)
	Divider:SetScript("OnHide", function() 
						if ( this.isMoving ) then
							this:StopMovingOrSizing();
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+1,0)
							this.isMoving = false;
						 end
						end)
	Divider:SetScript("OnUpdate", function() 
						if ( this.isMoving ) then
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+1,0)
						end
						end)	
end

function Table:AddColumn(Name,TableEntry,Abbrev)
	local Column=CreateFrame("FRAME",nil,self)
	Column:SetWidth(100)
	Column:SetHeight(self:GetHeight()-2)

	Column.Title=CreateFrame("FRAME",nil,Column)
	Column.Title:SetHeight(Relog.db.profile.RowHeight)
	Column.Title:SetPoint("TOPLEFT",Column,"TOPLEFT")
	Column.Title:SetPoint("TOPRIGHT",Column,"TOPRIGHT")

	Column.Title.Text=Column.Title:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Column.Title.Text:SetTextColor(1.0,1.0,1.0)
	Column.Title.Text:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
	Column.Title.Text:SetPoint("CENTER",Column.Title,"CENTER")
	Column.Title.Text:SetText(Name)

	Column.Title.Background=Column.Title:CreateTexture(nil,"BACKGROUND")
	Column.Title.Background:SetAllPoints(Column.Title)

	if math.fmod(#self.Columns,2)==0 then
		Column.Title.Background:SetTexture(0,0,0,0.4)
	else
		Column.Title.Background:SetTexture(0.2,0.2,0.2,0.4)
	end

	if #self.Columns==0 then
		Column:SetPoint("TOPLEFT",self,"TOPLEFT",1,-1)
	else
		local Divider=CreateFrame("FRAME",nil,self)
		Divider:SetWidth(2)
		Divider:SetHeight(self:GetHeight()-2)
		Divider:SetPoint("CENTER",self,"LEFT",10,0)

		Divider.Background=Divider:CreateTexture(nil,"BACKGROUND")
		Divider.Background:SetAllPoints(Divider)
		Divider.Background:SetTexture(1.0,1.0,1.0)
		Table:SetupDivider(Divider)


		self.Dividers[#self.Dividers+1]=Divider

		self.Columns[#self.Columns]:SetPoint("BOTTOMRIGHT",Divider,"BOTTOMLEFT")
		Column:SetPoint("TOPLEFT",Divider,"TOPRIGHT")
	end
	Column:SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-1,1)

	self.Columns[#self.Columns+1]=Column
	self:UpdateDividers()
end

function Table:UpdateDividers()
	local DividerNum=#self.Dividers
	local Spacing=(self:GetWidth()-2)/(DividerNum+1)

	for k, v in pairs(self.Dividers) do
		v:SetPoint("CENTER",self,"LEFT",1+k*Spacing,0)
	end
end

function Table:CreateTable()
	local theFrame=CreateFrame("FRAME",nil,UIParent)

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(172)
	theFrame:SetWidth(400)

	theFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	})
	theFrame:SetBackdropBorderColor(1.0,1.0,1.0)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255)
	
	theFrame:EnableMouse(true)
	theFrame:SetMovable(true)
	theFrame:SetResizable(true)
	
	theFrame:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
	theFrame:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
	theFrame:SetScript("OnHide", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)	
	theFrame.Columns={}
	theFrame.Dividers={}
	theFrame.AddColumn=Table.AddColumn
	theFrame.UpdateDividers=Table.UpdateDividers

	return theFrame
end

