Relog = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0","Parser-3.0")

local DefaultConfig={
	--Abbreviations reduce space stored for the log
	Abbrev={},	--Table converts from string to abbreviation (a number)
	RAbbrev={},	--Table converts from abbreviation to string

	--The parsed combat log
	LogOffset=0,
	StartLog=1,
	LogSize=0,
	NextLog=1,
	Log={
		Time={},
		Source={},
		Victim={},
		Type={},	--What type of log message is this? ("Damage" is most common so default)
		Ability={},	--false is most common so set as default
		Result={},	--"Hit" is most common so used as default
		Amount={},

		--Special Data for Dispel
		VictimAbility={},

		--Special Data for Drain/Gain
		Attribute={},

		--Leech Info
		SourceGained={},
		AttributeGained={},
		AmountGained={},

		--Reputation
		Faction={},
		Rank={},

		--Special Data for Damage
		DamageType={},
		AbsorbAmount={['*']=0},
		BlockAmount={['*']=0},
		ResistAmount={['*']=0},
		VulnerableAmount={['*']=0},

		--Healing
		Overheal={},

		--Item Data
		ItemLink={},
		ItemName={},
		ItemRarity={},
		ItemString={},

		--Items just for Experience which requires several
		BonusAmount={['*']=0},
		BonusType={},
		PenaltyAmount={['*']=0},
		PenaltyType={},
		AmountRaidPenalty={['*']=0},
		AmountGroupBonus={['*']=0},
		CompiledCache={},
	},

	--Units and their time based data
	Units={['*']={
			LastUpdated=0,
			
			--Attributes
			UnitID={},
			Friendly={},
			isPlayer={},
			isGrouped={},
			lClass={},
			Class={},
			Faction={},
			Level={},
			
			--Stats
			Health={},
			HealthMax={},
			Power={},
			PowerMax={},
			PowerType={},
			MinRange={},
			MaxRange={},

			--PvP (use with GetPVPRankInfo and UnitPVPRank)
			RankName={},
			RankNumber={},

			Last={
				UnitID,
				Friendly,
				isPlayer,
				isGrouped,
				lClass,
				Class,
				Faction,
				Level,
				
				--Stats
				Health,
				HealthMax,
				Power,
				PowerMax,
				PowerType,
				MinRange,
				MaxRange,

				--PvP (use with GetPVPRankInfo and UnitPVPRank)
				RankName,
				RankNumber,
			}
		}
	}
}

local ProfileDefaults={
	MaxLogs=10000,	
	RowHeight=12,
	Font="Arial Narrow",
}

Relog:RegisterDB("RelogDB","RelogPerCharDB")
Relog:RegisterDefaults('char',DefaultConfig)	
Relog:RegisterDefaults('profile',ProfileDefaults)	




--Subsecond determining by enclosing a range at second flips
Relog.SubLower=0
Relog.SubWidth=2

local AcceptedWidth=0.0025

function Relog:DetermineSubsecond()
	local UpperTime=GetTime()
	local CurTime=time()

	if (CurTime-Relog.SubLastTime2)==1 then
		--1 second difference
		local LowerTime=Relog.SubLastTime
		local Width=UpperTime-LowerTime
		LowerTime=LowerTime-math.floor(LowerTime)

		if Relog.SubLower<LowerTime then
			Relog.SubWidth=Relog.SubWidth-(LowerTime-Relog.SubLower)
			Relog.SubLower=LowerTime

			--Relog:Print("Raised Lower to "..LowerTime)
		end

		if (Relog.SubWidth+Relog.SubLower)>(Width+LowerTime) then
			Relog.SubWidth=Width+LowerTime-Relog.SubLower
			--Relog:Print("Reduced Width to "..Relog.SubWidth)
		end

		if Relog.SubWidth<AcceptedWidth then
			--Relog:Print("Finished with range of ".. Relog.SubLower .." to ".. Relog.SubLower+Relog.SubWidth)
			Relog:CancelScheduledEvent("DetermineSubSecond")
		end
	end
	
	Relog.SubLastTime=UpperTime
	Relog.SubLastTime2=CurTime
end

function Relog:GetSubsecond()
	local T=GetTime()
	T=T-Relog.SubLower+Relog.SubWidth/2
	T=T-math.floor(T)
	return T
end

function Relog:GetTime()
	return time()+Relog:GetSubsecond()
end



function Relog:OnInitialize()
	Relog.SubLastTime=GetTime()
	Relog.SubLastTime2=time()

	Relog:ScheduleRepeatingEvent("DetermineSubSecond",Relog.DetermineSubsecond,0)
	Relog.Window:CreateWindow()
end

function Relog:OnEnable(first)
	--Relog:ResetDB()
	Relog.Parse.SetupParser()
end

function Relog:ResetData()
	Relog.db.char.Log={}
	Relog.db.char.StartLog=1
	Relog.db.char.NextLog=1
	Relog.db.char.LogSize=0
	Relog.db.char.Units={}
	Relog.db.char.Abrrev={}
	Relog.db.char.RAbrrev={}
	Relog.Window:RefreshRows()
end


function Relog:ChangeMaxLogs(NewVal)
	local OldVal=Relog.db.profile.MaxLogs
	Relog.db.profile.MaxLogs=NewVal

	local Start=Relog.db.char.StartLog
	local Size=Relog.db.char.LogSize

	if NewVal>OldVal and (Start+Size)>=OldVal then
		local NewPos=NewVal
		for i=OldVal,Start,-1 do
			for k,v in pairs(Relog.db.char.Log) do 
				v[NewPos]=v[i]
				v[i]=nil
			end
			NewPos=NewPos-1
		end
		Relog.db.char.StartLog=Relog.db.char.StartLog+NewVal-OldVal
	elseif NewVal<OldVal then
		local NewStart=Start-(OldVal-NewVal)

		if NewStart<1 then
			NewStart=NewStart+NewVal
		end

		Relog.db.char.StartLog=NewStart

		local NewPos=NewStart
		local OldPos=Start
		
		if Relog.db.char.LogSize>NewVal then
			OldPos=OldPos+(Relog.db.char.LogSize-NewVal)
		end
		if OldPos>OldVal then
			OldPos=OldPos-OldVal
		end

		for i=1,NewVal do
			for k, v in pairs(Relog.db.char.Log) do 
				v[NewPos]=v[OldPos]
				v[OldPos]=nil
			end
			NewPos=NewPos+1
			OldPos=OldPos+1
			if NewPos>NewVal then
				NewPos=NewPos-NewVal
			end
			if OldPos>OldVal then
				OldPos=OldPos-OldVal
			end
		end

		for i=NewVal+1,OldVal do
			for k,v in pairs(Relog.db.char.Log) do 
				v[i]=nil
			end
		end

		if Relog.db.char.LogSize>NewVal then
			Relog.db.char.LogSize=NewVal
		end
	end
end